<?php

namespace Model;

use API\script\Gateway\GatewayForm;
use API\script\Gateway\GatewayKeyword;
use API\script\Gateway\GatewayListResponseOfCandidate;
use API\script\Gateway\GatewayPossibleResponse;
use API\script\Gateway\GatewayQuestion;
use API\script\Gateway\GatewayResponse;
use BusinessClass\Form;
use PDOException;

/**
 * Permet de développer les fonctions appelées par le controllerAdmin pour gérer
 * les actions de l'administrateur
 */
class ModelAdmin
{
    public function goToAdmin(): void
    {
        global $rep, $views;
        try{
            require_once($rep . $views['admin']);
        } catch (PDOException $e) {
            $error = $e->getMessage();
            require_once($rep . $views['form']);
    }
}
    /**
     * Permet de créer et d'ajouter une question et de retourner son ID afin de la
     * reconnaitre facilement dans la suite du code.
     *
     * @return int
     */
    public function addQuestion(): int
    {
        $questionContent = $_POST['question'];
        $type = $_POST['type'];

        $question = new $type(0, $questionContent);

        $form = (new GatewayForm())->getForm();
        if (!empty($form)) {
            return (new GatewayQuestion())->addQuestion(array(get_class($question) ,$question->getContent()), $form[0]['id']);
        }

        return -1;
    }

    /**
     * Permet de supprimer une question du formulaire
     *
     * @return void
     */
    public function deleteQuestion():void
    {
        $idQuestion = $_POST["idQuestion"];
        $type = $_POST["type"];
        (new GatewayQuestion())->deleteQuestion($idQuestion, $type);
    }

    /**
     * Permet d'ajouter une possibilité de réponse à une question en l'assignant à des catégories.
     *
     * @return void
     */
    public function addResponse(): void
    {
        $idQuestion = $_POST['idQuestion'];
        $response = $_POST['response'];
        $categories = $_POST['categories'];
        if ($categories == null) {
            $categories = [];
        }
        (new GatewayQuestion())->insertResponseInQuestion($response, $categories, $idQuestion);
    }

    /**
     * Permet de supprimer une possible réponse à une question
     *
     * @return void
     */
    public function deleteResponse(): void
    {
        (new GatewayPossibleResponse())->deletePossibleResponse($_POST["possibleResponse"]);
    }

    /**
     * Permet de créer un nouveau formulaire en précisant son titre et sa description.
     *
     * @return void
     */
    public function createForm(): void
    {
        if (empty((new GatewayForm())->getForm())) {
            $form = new Form(0, "Votre avis nous intéresse !!!", "Description de notrer formulaire", array());
            (new GatewayForm())->insertForm($form);
        }
    }


    /**
     * Permet d'ajouter une nouvelle catégorie (mot-clef)
     *
     * @return void
     */
    public function addKeyword(): void
    {
        $keyword = $_POST['keyword'];
        (new GatewayKeyword())->insertKeyword($keyword);
    }

    /**
     * Permet de supprimer une catégorie (mot-clef)
     *
     * @return void
     */
    public function deleteKeyword(): void
    {
        (new GatewayKeyword())->deleteKeyword($_POST["idCateg"]);
    }

    /**
     * Permet de récupérer toutes les catégories existantes.
     *
     * @return array
     */
    public function getCategories(): array
    {
        $categories = [];
        foreach ((new GatewayKeyword())->getAllKeyword() as $category) {
            $categories[] = $category["word"];
        }

        return $categories;
    }


    /**
     * Permet de récupérer toutes les questions existantes.
     *
     * @return array
     */
    public function getQuestions(): array
    {
        if((new GatewayForm())->existsForm()) {
            $idForm = (new GatewayForm())->getForm()[0]["id"];
            $questionsArray = (new GatewayQuestion())->getAllQuestions($idForm);
            return Factory::getBuiltObjects($questionsArray, "Question");
        }
        else {
            return array();
        }
    }


    /**
     * Permet de récupérer toutes les réponses existantes.
     *
     * @return array
     */
    public function getResponsesCandidate(): array
    {
        $responsesCandidate = (new GatewayListResponseOfCandidate())->getAllListResponsesOfCandidate();
        $results = [];
        foreach ($responsesCandidate as $response) {
            $results[] = (new GatewayListResponseOfCandidate())->getDetailsListResponsesOfCandidate($response["id"]);
        }

        return $results;
    }

    /**
     * Permet de supprimer les réponses d'une personne d'un formulaire
     *
     * @return void
     */
    public function deleteResponsesCandidate(): void
    {
        (new GatewayListResponseOfCandidate())->deleteListResponseOfCandidate($_POST["idResponseCandidate"]);
    }


    /**
     * Permet de récupérer tous les profiles intéressants à interviewer
     *
     * @return array
     */
    public function getInterestingProfiles(): array
    {
        $occurences = ["2ème année" => 3, "Stage" => 3, "Alternance" => 1];
        $responsesCandidates = (new GatewayListResponseOfCandidate())->getAllListResponsesOfCandidate();
        $responses = [];
        $profil = [];
        foreach ($responsesCandidates as $candidate) {
            $responses = (new GatewayListResponseOfCandidate())->getDetailsListResponsesOfCandidate($candidate["id"]);
            $tab = [];
            foreach ($responses[2] as $categories) {
                if (isset($categories[0])) {
                    $tab[] = $categories[0];
                }
            }
            $profil[$candidate["id"]] = $tab;
        }

        foreach ($profil as $key => $elements) {
            foreach ($elements as $profilElement) {
                if (!array_key_exists($profilElement, $occurences)) {
                    var_dump($key);
                    var_dump($elements);
                    echo "----------------";
                    $index = array_search($profilElement, $elements);
                    if ($index !== false) {
                        unset($elements[$index]);
                    }
                }
            }
        }

        echo "\ndfoeznfiozerbgiofzerjgopizerjgioerhj\n";

        foreach ($profil as $key => $elements) {
            foreach ($elements as $profilElement) {
                if ($occurences[$profilElement] <= min($occurences)) {
                    var_dump($key);
                    var_dump($profilElement);
                }
            }
        }

        return [];
    }

}
